@@ -6,15 +6,15 @@ import json
 import urllib.parse
 import urllib.request
 import webbrowser
-from typing import Dict
+from typing import Dict, Any
 
 from .checker import *
-from .database import MongoDBHandler
-from .helper import handle_dot_in_keys
+from .helper import Waiter, credentials_file
+from .hubapi import _list, _push, _list_local
 from ..clients.python import ProgressBar
-from ..excepts import PeaFailToStart
+from ..excepts import PeaFailToStart, TimedOutException, DockerLoginFailed
 from ..helper import colored, get_readable_size, get_now_timestamp, get_full_version, random_name, expand_dict
-from ..logging import get_logger
+from ..logging import JinaLogger
 from ..logging.profile import TimeContext
 
 if False:
@@ -39,8 +39,11 @@ class HubIO:
     """
 
     def __init__(self, args: 'argparse.Namespace'):
-        self.logger = get_logger(self.__class__.__name__, **vars(args))
+        self.logger = JinaLogger(self.__class__.__name__, **vars(args))
         self.args = args
+        self._load_docker_client()
+
+    def _load_docker_client(self):
         try:
             import docker
             from docker import APIClient
@@ -50,24 +53,24 @@ class HubIO:
             # low-level client
             self._raw_client = APIClient(base_url='unix://var/run/docker.sock')
         except (ImportError, ModuleNotFoundError):
-            self.logger.critical('requires "docker" dependency, please install it via "pip install jina[docker]"')
-            raise
+            self.logger.warning('missing "docker" dependency, available CLIs limited to "jina hub [list, new]"'
+                                'to enable full CLI, please do pip install "jina[docker]"')
 
     def new(self) -> None:
         """Create a new executor using cookiecutter template """
         try:
             from cookiecutter.main import cookiecutter
         except (ImportError, ModuleNotFoundError):
-            self.logger.critical('requires "cookiecutter" dependency, please install it via "pip install cookiecutter"')
+            self.logger.critical('"jina hub new" requires "cookiecutter" dependency, '
+                                 'please install it via pip install "jina[cookiecutter]"')
             raise
 
-        import click
+        import click  # part of cookiecutter
         cookiecutter_template = self.args.template
         if self.args.type == 'app':
             cookiecutter_template = 'https://github.com/jina-ai/cookiecutter-jina.git'
         elif self.args.type == 'pod':
             cookiecutter_template = 'https://github.com/jina-ai/cookiecutter-jina-hub.git'
-        cookiecutter(cookiecutter_template, overwrite_if_exists=self.args.overwrite, output_dir=self.args.output_dir)
 
         try:
             cookiecutter(cookiecutter_template, overwrite_if_exists=self.args.overwrite,
@@ -75,6 +78,82 @@ class HubIO:
         except click.exceptions.Abort:
             self.logger.info('nothing is created, bye!')
 
+    def login(self) -> None:
+        """Login using Github Device flow to allow push access to Jina Hub Registry"""
+        import requests
+
+        with resource_stream('jina', '/'.join(('resources', 'hubapi.yml'))) as fp:
+            hubapi_yml = yaml.load(fp)
+
+        client_id = hubapi_yml['github']['client_id']
+        scope = hubapi_yml['github']['scope']
+        device_code_url = hubapi_yml['github']['device_code_url']
+        access_token_url = hubapi_yml['github']['access_token_url']
+        grant_type = hubapi_yml['github']['grant_type']
+        seconds_to_wait = hubapi_yml['github']['wait_time_for_access_token']
+
+        headers = {'Accept': 'application/json'}
+        code_request_body = {
+            'client_id': client_id,
+            'scope': scope
+        }
+        try:
+            self.logger.info('logging in via Github device flow!')
+            response = requests.post(url=device_code_url,
+                                     headers=headers,
+                                     data=code_request_body)
+            if response.status_code != requests.codes.ok:
+                self.logger.error('cannot reach github server. please make sure you\'re connected to internet')
+
+            code_response = response.json()
+            device_code = code_response['device_code']
+            user_code = code_response['user_code']
+            verification_uri = code_response['verification_uri']
+
+            self.logger.info(f'please go to {colored(verification_uri, "cyan", attrs=["underline"])} & enter code '
+                             f'{colored(user_code, "cyan", attrs=["bold"])} to authorize jina to login via Github OAuth')
+            access_request_body = {
+                'client_id': client_id,
+                'device_code': device_code,
+                'grant_type': grant_type
+            }
+
+            with Waiter(seconds=seconds_to_wait, message='to fetch access token') as waiter:
+                while True:
+                    response = requests.post(url=access_token_url,
+                                             headers=headers,
+                                             data=access_request_body)
+                    access_token_response = response.json()
+                    if waiter.is_time_up:
+                        raise TimedOutException(f'login operation failed. '
+                                                f'waited for {seconds_to_wait} seconds before timing out.')
+                    if 'error' in access_token_response and access_token_response['error'] == 'authorization_pending':
+                        waiter.sleep(5)
+                    if 'access_token' in access_token_response:
+                        token = {
+                            'access_token': access_token_response['access_token']
+                        }
+                        with open(credentials_file(), 'w') as cf:
+                            yaml.dump(token, cf)
+                        self.logger.info(f'successfully logged in!')
+                        break
+
+        except KeyError as exp:
+            self.logger.error(f'can not read the key in response: {exp}')
+        except Exception as exp:
+            self.logger.error(f'login failed: {exp}')
+
+    def list(self) -> Dict[str, Any]:
+        """ List all hub images given a filter specified by CLI """
+        if self.args.local_only:
+            return _list_local(self.logger)
+        else:
+            return _list(logger=self.logger,
+                         image_name=self.args.name,
+                         image_kind=self.args.kind,
+                         image_type=self.args.type,
+                         image_keywords=self.args.keywords)
+
     def push(self, name: str = None, readme_path: str = None) -> None:
         """ A wrapper of docker push 
         - Checks for the tempfile, returns without push if it cannot find
@@ -84,28 +163,31 @@ class HubIO:
         name = name or self.args.name
         file_path = get_summary_path(name)
         if not os.path.isfile(file_path):
-            self.logger.error(f'can not find the build summary file')
+            self.logger.error(f'can not find the build summary file. '
+                              f'please build the image before pushing and pass tag with the image name')
             return
 
         try:
             self._push_docker_hub(name, readme_path)
-        except:
-            self.logger.error('can not push to the docker hub registry')
+        except Exception as ex:
+            self.logger.error(f'can not complete the push due to {repr(ex)}')
             return
 
         with open(file_path) as f:
             result = json.load(f)
         if result['is_build_success']:
-            self._write_summary_to_db(summary=result)
+            _push(logger=self.logger, summary=result)
 
     def _push_docker_hub(self, name: str = None, readme_path: str = None) -> None:
         """ Helper push function """
         check_registry(self.args.registry, name, _repo_prefix)
         self._check_docker_image(name)
-        self.login()
+        self._docker_login()
         with ProgressBar(task_name=f'pushing {name}', batch_unit='') as t:
             for line in self._client.images.push(name, stream=True, decode=True):
                 t.update(1)
+                if 'error' in line and 'authentication required' in line['error']:
+                    raise DockerLoginFailed('user not logged in to docker.')
                 self.logger.debug(line)
         self.logger.success(f'🎉 {name} is now published!')
 
@@ -137,8 +219,8 @@ class HubIO:
     def pull(self) -> None:
         """A wrapper of docker pull """
         check_registry(self.args.registry, self.args.name, _repo_prefix)
-        self.login()
         try:
+            self._docker_login()
             with TimeContext(f'pulling {self.args.name}', self.logger):
                 image = self._client.images.pull(self.args.name)
             if isinstance(image, list):
@@ -146,10 +228,8 @@ class HubIO:
             image_tag = image.tags[0] if image.tags else ''
             self.logger.success(
                 f'🎉 pulled {image_tag} ({image.short_id}) uncompressed size: {get_readable_size(image.attrs["Size"])}')
-        except:
-            self.logger.error(f'can not pull image {self.args.name} from {self.args.registry}')
-            raise
-
+        except Exception as ex:
+            self.logger.error(f'can not pull image {self.args.name} from {self.args.registry} due to {repr(ex)}')
 
     def _check_docker_image(self, name: str) -> None:
         # check local image
@@ -168,13 +248,18 @@ class HubIO:
 
         self.logger.info(f'✅ {name} is a valid Jina Hub image, ready to publish')
 
-    def login(self) -> None:
+    def _docker_login(self) -> None:
         """A wrapper of docker login """
+        from docker.errors import APIError
         if self.args.username and self.args.password:
-            self._client.login(username=self.args.username, password=self.args.password,
-                               registry=self.args.registry)
+            try:
+                self._client.login(username=self.args.username, password=self.args.password,
+                                   registry=self.args.registry)
+                self.logger.debug(f'successfully logged in to docker hub')
+            except APIError:
+                raise DockerLoginFailed(f'invalid credentials passed. docker login failed')
         else:
-            raise ValueError('no username/password specified, docker login failed')
+            raise DockerLoginFailed('no username/password specified, docker login failed')
 
     def build(self) -> Dict:
         """A wrapper of docker build """
@@ -240,16 +325,21 @@ class HubIO:
 
             if is_build_success:
                 if self.args.test_uses:
+                    from jina.excepts import PretrainedModelFileDoesNotExist
                     try:
                         is_build_success = False
                         from jina.flow import Flow
                         p_name = random_name()
                         with Flow().add(name=p_name, uses=image.tags[0], daemon=self.args.daemon):
                             pass
+
                         if self.args.daemon:
                             self._raw_client.stop(p_name)
                         self._raw_client.prune_containers()
                         is_build_success = True
+                    except PretrainedModelFileDoesNotExist:
+                        self.logger.warning(f' Pretrained Model File Does not Exist is considered as a test passing')
+                        is_build_success = True
                     except PeaFailToStart:
                         self.logger.error(f'can not use it in the Flow, please check your file bundle')
                     except Exception as ex:
@@ -292,7 +382,7 @@ class HubIO:
                 if self.args.push:
                     try:
                         self._push_docker_hub(image.tags[0], self.readme_path)
-                        self._write_summary_to_db(summary=result)
+                        _push(logger=self.logger, summary=result)
                         self._write_slack_message(result, _details, _build_history)
                     except Exception as ex:
                         self.logger.error(f'can not complete the push due to {repr(ex)}')
@@ -313,30 +403,6 @@ class HubIO:
                  'exception': str(ex)}
         return s
 
-    def _write_summary_to_db(self, summary: Dict) -> None:
-        """ Inserts / Updates summary document in mongodb """
-        if not is_db_envs_set():
-            self.logger.warning('MongoDB environment vars are not set! bookkeeping skipped.')
-            return
-
-        build_summary = handle_dot_in_keys(document=summary)
-        _build_query = {'name': build_summary['name'], 'version': build_summary['version']}
-        _current_build_history = build_summary['build_history']
-        with MongoDBHandler(hostname=os.environ['JINA_DB_HOSTNAME'],
-                            username=os.environ['JINA_DB_USERNAME'],
-                            password=os.environ['JINA_DB_PASSWORD'],
-                            database_name=os.environ['JINA_DB_NAME'],
-                            collection_name=os.environ['JINA_DB_COLLECTION']) as db:
-            existing_doc = db.find(query=_build_query)
-            if existing_doc:
-                build_summary['build_history'] = existing_doc['build_history'] + _current_build_history
-                _modified_count = db.replace(document=build_summary,
-                                             query=_build_query)
-                self.logger.debug(f'Updated the build + push summary in db. {_modified_count} documents modified')
-            else:
-                _inserted_id = db.insert(document=build_summary)
-                self.logger.debug(f'Inserted the build + push summary in db with id {_inserted_id}')
-
     def _write_summary_to_file(self, summary: Dict) -> None:
         file_path = get_summary_path(f'{summary["name"]}:{summary["version"]}')
         with open(file_path, 'w+') as f:
