@@ -152,3 +152,28 @@ def guess_mime(uri):
         tmp = urllib.request.urlopen(page)
         m_type = tmp.info().get_content_type()
     return m_type
+
+
+class DocGroundtruthPair:
+    """
+    Helper class to expose common interface to the traversal logic of the BaseExecutable Driver.
+    It is important to note that it checks the matching structure of `docs` and `groundtruths`. It is important while
+    traversing to ensure that then the driver can be applied at a comparable level of granularity and adjacency.
+    This does not imply that you can't compare at the end a document with 10 matches with a groundtruth with 20 matches
+    """
+
+    def __init__(self, doc: 'jina_pb2.Document', groundtruth: 'jina_pb2.Document'):
+        self.doc = doc
+        self.groundtruth = groundtruth
+
+    @property
+    def matches(self):
+        assert self.groundtruth and len(self.doc.matches) == len(self.groundtruth.matches)
+        for doc, groundtruth in zip(self.doc.matches, self.groundtruth.matches):
+            yield DocGroundtruthPair(doc, groundtruth)
+
+    @property
+    def chunks(self):
+        assert self.groundtruth and len(self.doc.chunks) == len(self.groundtruth.chunks)
+        for doc, groundtruth in zip(self.doc.chunks, self.groundtruth.chunks):
+            yield DocGroundtruthPair(doc, groundtruth)
