@@ -1,6 +1,7 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+import sys
 from typing import Iterable
 
 from .. import QuerySetReader, BaseRecursiveDriver
@@ -19,14 +20,13 @@ class SliceQL(QuerySetReader, BaseRecursiveDriver):
         - !SortQL
             with:
                 reverse: true
-                field: 'score.value'
+                field: 'score__value'
                 traversal_paths: ['m']
         - !SliceQL
             with:
                 start: 0
                 end: 50
-                granularity_range: [0, 0]
-                adjacency_range: [0, 1]
+                traversal_paths: ['m']
 
         `SliceQL` will ensure that only the first 50 documents are returned from this `Pod`
     """
@@ -41,8 +41,10 @@ class SliceQL(QuerySetReader, BaseRecursiveDriver):
         """
         super().__init__(*args, **kwargs)
         self._start = int(start)
-        self._end = int(end)
-        self.is_apply = False
+        if end is None:
+            self._end = sys.maxsize
+        else:
+            self._end = int(end)
 
     def _apply_all(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs) -> None:
         if self.start <= 0 and (self.end is None or self.end >= len(docs)):
