@@ -1,10 +1,10 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-from typing import Iterable, List
+from typing import Iterable, Tuple
 
+from .queryset.dunderkey import dunder_get
 from .. import QuerySetReader, BaseRecursiveDriver
-from ...helper import rgetattr
 
 if False:
     from ...proto import jina_pb2
@@ -21,29 +21,26 @@ class SortQL(QuerySetReader, BaseRecursiveDriver):
         - !SortQL
             with:
                 reverse: true
-                field: 'score.value'
+                field: 'score__value'
                 traversal_paths: ['m']
         - !SliceQL
             with:
                 start: 0
                 end: 50
-                granularity_range: [0, 0]
-                adjacency_range: [0, 1]
+                traversal_paths: ['m']
 
         `SortQL` will ensure that only the documents are sorted by the score value before slicing the first top 50 documents
     """
 
-    def __init__(self, field: str, reverse: bool = False, traversal_paths: List[str] = ['c'], *args, **kwargs):
+    def __init__(self, field: str, reverse: bool = False, traversal_paths: Tuple[str] = ('c',), *args, **kwargs):
         """
         :param field: the value of the field drives the sort of the iterable docs
         :param reverse: sort the value from big to small
         """
 
-        super().__init__(use_tree_traversal=True, traversal_paths=traversal_paths, *args, **kwargs)
+        super().__init__(traversal_paths=traversal_paths, *args, **kwargs)
         self._reverse = reverse
         self._field = field
-        self.is_apply = False
 
     def _apply_all(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs) -> None:
-        docs.sort(key=lambda x: rgetattr(x, self.field), reverse=self.reverse)
-
+        docs.sort(key=lambda x: dunder_get(x, self.field), reverse=self.reverse)
