@@ -1,30 +1,33 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-from typing import Iterable, List
+from typing import Iterable, Tuple
 
 from . import BaseExecutableDriver, QuerySetReader
 from .helper import extract_docs, array2pb
-
-if False:
-    from ..proto import jina_pb2
+from ..proto import uid, jina_pb2
 
 
 class BaseSearchDriver(BaseExecutableDriver):
     """Drivers inherited from this Driver will bind :meth:`craft` by default """
 
     def __init__(
-        self,
-        executor: str = None,
-        method: str = 'query',
-        traversal_paths: List[str] = ['r', 'c'],
-        *args,
-        **kwargs
-    ):
-        super().__init__(executor, method, traversal_paths=traversal_paths, *args, **kwargs)
-        self._is_apply = False
-        # search driver recursion apply in pre-order
-        self.recursion_order = 'pre'
+            self,
+            executor: str = None,
+            method: str = 'query',
+            traversal_paths: Tuple[str] = ('r', 'c'),
+            *args,
+            **kwargs):
+        super().__init__(
+            executor,
+            method,
+            traversal_paths=traversal_paths,
+            *args,
+            **kwargs
+        )
+
+        self.hash2id = uid.hash2id
+        self.id2hash = uid.id2hash
 
 
 class KVSearchDriver(BaseSearchDriver):
@@ -56,8 +59,11 @@ class KVSearchDriver(BaseSearchDriver):
     def _apply_all(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs) -> None:
         miss_idx = []  #: missed hit results, some search may not end with results. especially in shards
         for idx, retrieved_doc in enumerate(docs):
-            r = self.exec_fn(retrieved_doc.id)
-            if r:
+            serialized_doc = self.exec_fn(self.id2hash(retrieved_doc.id))
+            if serialized_doc:
+                r = jina_pb2.Document()
+                r.ParseFromString(serialized_doc)
+
                 # TODO: this isn't perfect though, merge applies recursively on all children
                 #  it will duplicate embedding.shape if embedding is already there
                 if self._is_merge:
@@ -66,7 +72,6 @@ class KVSearchDriver(BaseSearchDriver):
                     retrieved_doc.CopyFrom(r)
             else:
                 miss_idx.append(idx)
-
         # delete non-existed matches in reverse
         for j in reversed(miss_idx):
             del docs[j]
@@ -80,7 +85,7 @@ class VectorFillDriver(QuerySetReader, BaseSearchDriver):
         super().__init__(executor, method, *args, **kwargs)
 
     def _apply_all(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs) -> None:
-        embeds = self.exec_fn([d.id for d in docs])
+        embeds = self.exec_fn([self.id2hash(d.id) for d in docs])
         for doc, embedding in zip(docs, embeds):
             doc.embedding.CopyFrom(array2pb(embedding))
 
@@ -106,25 +111,27 @@ class VectorSearchDriver(QuerySetReader, BaseSearchDriver):
     def _apply_all(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs) -> None:
         embed_vecs, doc_pts, bad_doc_ids = extract_docs(docs, embedding=True)
 
+        if not doc_pts:
+            return
+
         fill_fn = getattr(self.exec, 'query_by_id', None)
         if self._fill_embedding and not fill_fn:
             self.logger.warning(f'"fill_embedding=True" but {self.exec} does not have "query_by_id" method')
 
-        if doc_pts:
-            if bad_doc_ids:
-                self.logger.warning(f'these bad docs can not be added: {bad_doc_ids}')
-            idx, dist = self.exec_fn(embed_vecs, top_k=int(self.top_k))
-            op_name = self.exec.__class__.__name__
-            for doc, topks, scores in zip(doc_pts, idx, dist):
+        if bad_doc_ids:
+            self.logger.warning(f'these bad docs can not be added: {bad_doc_ids}')
+        idx, dist = self.exec_fn(embed_vecs, top_k=int(self.top_k))
+        op_name = self.exec.__class__.__name__
+        for doc, topks, scores in zip(doc_pts, idx, dist):
 
-                topk_embed = fill_fn(topks) if (self._fill_embedding and fill_fn) else [None] * len(topks)
+            topk_embed = fill_fn(topks) if (self._fill_embedding and fill_fn) else [None] * len(topks)
 
-                for match_id, score, vec in zip(topks, scores, topk_embed):
-                    r = doc.matches.add()
-                    r.id = match_id
-                    r.adjacency = doc.adjacency + 1
-                    r.score.ref_id = doc.id
-                    r.score.value = score
-                    r.score.op_name = op_name
-                    if vec is not None:
-                        r.embedding.CopyFrom(array2pb(vec))
+            for match_hash, score, vec in zip(topks, scores, topk_embed):
+                r = doc.matches.add()
+                r.id = self.hash2id(match_hash)
+                r.adjacency = doc.adjacency + 1
+                r.score.ref_id = doc.id
+                r.score.value = score
+                r.score.op_name = op_name
+                if vec is not None:
+                    r.embedding.CopyFrom(array2pb(vec))
