@@ -1,6 +1,3 @@
-__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
-__license__ = "Apache-2.0"
-
 """
 Miscellaneous enums used in jina
 
@@ -35,6 +32,9 @@ To use these enums in YAML config, following the example below:
         parallel_type: any
 """
 
+__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
+__license__ = "Apache-2.0"
+
 from enum import IntEnum, EnumMeta
 
 
@@ -205,11 +205,31 @@ class PeaRoleType(BetterEnum):
     """ The enum of a Pea role
 
     """
-    REPLICA = 0
+    SINGLETON = 0
     HEAD = 1
     TAIL = 2
-    SHARD = 3
-    SINGLETON = 4
+    PARALLEL = 3
+
+
+class PodRoleType(BetterEnum):
+    """ The enum of a Pod role for visualization
+
+    """
+    POD = 0
+    JOIN = 1
+    INSPECT = 2
+    GATEWAY = 3
+    INSPECT_AUX_PASS = 4
+    JOIN_INSPECT = 5
+
+    @property
+    def is_inspect(self) -> bool:
+        """
+        if the role is inspect pod related
+
+        :return:
+        """
+        return self.value in {2, 4}
 
 
 class ClientMode(BetterEnum):
@@ -219,6 +239,7 @@ class ClientMode(BetterEnum):
     INDEX = 0
     SEARCH = 1
     TRAIN = 2
+    EVALUATE = 3
 
 
 class OnErrorSkip(BetterEnum):
@@ -231,3 +252,20 @@ class OnErrorSkip(BetterEnum):
     DRIVER = 2
     HANDLE = 3
     CALLBACK = 4
+
+
+class FlowInspectType(BetterEnum):
+    """ Inspect strategy in the flow
+    """
+
+    HANG = 0  # keep them hanging there
+    REMOVE = 1  # remove them in the build
+    COLLECT = 2  # spawn a new pod and collect them before build
+
+    @property
+    def is_keep(self) -> bool:
+        """
+
+        :return: if this socket is using `bind` protocol
+        """
+        return self.value in {0, 2}
\ No newline at end of file
