@@ -1,8 +1,8 @@
+""" This modules defines all kinds of exceptions raised in jina """
+
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-""" This modules defines all kinds of exceptions raised in jina """
-
 
 class NoExplicitMessage(Exception):
     """Waiting until all partial messages are received"""
@@ -102,3 +102,19 @@ class UndefinedModel(Exception):
 
 class MongoDBException(Exception):
     """ Any errors raised by MongoDb """
+
+
+class TimedOutException(Exception):
+    """ Errors raised for timeout operations """
+
+
+class DockerLoginFailed(Exception):
+    """ Exception to raise for docker hub login failures """
+
+
+class PretrainedModelFileDoesNotExist(Exception):
+    """ Exception to raise for executors depending on pretrained model files when they do not exist """
+
+
+class BadDocID(Exception):
+    """ Exception when user give a non-hex string as the doc id """
