@@ -15,15 +15,16 @@ from typing import Dict, Any, Union, TypeVar, Type, TextIO, List
 import ruamel.yaml.constructor
 from ruamel.yaml import StringIO
 
-from .decorators import as_train_method, as_update_method, store_init_kwargs
+from .decorators import as_train_method, as_update_method, store_init_kwargs, as_aggregate_method
 from .metas import get_default_metas, fill_metas_with_defaults
 from ..excepts import EmptyExecutorYAML, BadWorkspace, BadPersistantFile, NoDriverForRequest, UnattachedDriver
 from ..helper import yaml, PathImporter, expand_dict, expand_env_var, get_local_config_source
-from ..logging.base import get_logger
+from ..logging import JinaLogger
 from ..logging.profile import TimeContext
 
 if False:
     from ..drivers import BaseDriver
+    from ..peapods.pea import BasePea
 
 __all__ = ['BaseExecutor', 'AnyExecutor', 'ExecutorType']
 
@@ -60,9 +61,10 @@ class ExecutorType(type):
 
     @staticmethod
     def register_class(cls):
-        prof_funcs = ['train', 'encode', 'add', 'query', 'craft', 'score']
+        prof_funcs = ['train', 'encode', 'add', 'query', 'craft', 'score', 'evaluate']
         update_funcs = ['train', 'add']
         train_funcs = ['train']
+        aggregate_funcs = ['evaluate']
 
         def wrap_func(func_lst, wrapper):
             for f_name in func_lst:
@@ -77,6 +79,7 @@ class ExecutorType(type):
 
             wrap_func(train_funcs, as_train_method)
             wrap_func(update_funcs, as_update_method)
+            wrap_func(aggregate_funcs, as_aggregate_method)
 
             reg_cls_set.add(cls.__name__)
             setattr(cls, '_registered_class', reg_cls_set)
@@ -125,7 +128,11 @@ class BaseExecutor(metaclass=ExecutorType):
     store_args_kwargs = False  #: set this to ``True`` to save ``args`` (in a list) and ``kwargs`` (in a map) in YAML config
 
     def __init__(self, *args, **kwargs):
-        self.logger = get_logger(self.__class__.__name__)
+        if isinstance(args, tuple) and len(args) > 0:
+            self.args = args[0]
+        else:
+            self.args = args
+        self.logger = JinaLogger(self.__class__.__name__)
         self._snapshot_files = []
         self._post_init_vars = set()
         self._last_snapshot_ts = datetime.now()
@@ -296,7 +303,7 @@ class BaseExecutor(metaclass=ExecutorType):
 
     def __setstate__(self, d):
         self.__dict__.update(d)
-        self.logger = get_logger(self.__class__.__name__)
+        self.logger = JinaLogger(self.__class__.__name__)
         try:
             self._post_init_wrapper(fill_in_metas=False)
         except ImportError as ex:
@@ -445,7 +452,7 @@ class BaseExecutor(metaclass=ExecutorType):
         """
         Release the resources as executor is destroyed, need to be overrided
         """
-        pass
+        self.logger.close()
 
     def __enter__(self):
         return self
@@ -543,14 +550,18 @@ class BaseExecutor(metaclass=ExecutorType):
             r['metas'] = p
         return r
 
-    def attach(self, *args, **kwargs):
+    def attach(self, pea: 'BasePea', *args, **kwargs):
         """Attach this executor to a :class:`jina.peapods.pea.BasePea`.
 
         This is called inside the initializing of a :class:`jina.peapods.pea.BasePea`.
         """
         for v in self._drivers.values():
             for d in v:
-                d.attach(executor=self, *args, **kwargs)
+                d.attach(executor=self, pea=pea, *args, **kwargs)
+
+        # replacing the logger to pea's logger
+        if pea and isinstance(getattr(pea, 'logger', None), JinaLogger):
+            self.logger = pea.logger
 
     def __call__(self, req_type, *args, **kwargs):
         if req_type in self._drivers:
