@@ -3,6 +3,7 @@ __license__ = "Apache-2.0"
 
 import os
 
+from jina.excepts import PretrainedModelFileDoesNotExist
 from . import BaseEncoder
 from ..devices import OnnxDevice, PaddleDevice, TorchDevice, TFDevice, MindsporeDevice
 from ...helper import is_url
@@ -11,7 +12,7 @@ from ...helper import cached_property
 
 # mixin classes go first, base classes are read from right to left.
 class BaseOnnxEncoder(OnnxDevice, BaseEncoder):
-    def __init__(self, output_feature: str, model_path: str = None, *args, **kwargs):
+    def __init__(self, output_feature: str = None, model_path: str = None, *args, **kwargs):
         """
 
         :param output_feature: the name of the layer for feature extraction.
@@ -28,22 +29,27 @@ class BaseOnnxEncoder(OnnxDevice, BaseEncoder):
         Load the model from the `.onnx` file and add outputs for the selected layer, i.e. ``outputs_name``. The modified
              models is saved at `tmp_model_path`.
         """
-        import onnxruntime
         super().post_init()
-        self.model_name = self.raw_model_path.split('/')[-1]
-        self.tmp_model_path = self.get_file_from_workspace(f'{self.model_name}.tmp')
-        if is_url(self.raw_model_path):
+        model_name = self.raw_model_path.split('/')[-1] if self.raw_model_path else None
+        tmp_model_path = self.get_file_from_workspace(f'{model_name}.tmp') if model_name else None
+        raw_model_path = self.raw_model_path
+        if self.raw_model_path and is_url(self.raw_model_path):
             import urllib.request
             download_path, *_ = urllib.request.urlretrieve(self.raw_model_path)
-            self.raw_model_path = download_path
+            raw_model_path = download_path
             self.logger.info(f'download the model at {self.raw_model_path}')
-        if not os.path.exists(self.tmp_model_path):
-            self._append_outputs(self.raw_model_path, self.outputs_name, self.tmp_model_path)
-            self.logger.info(f'save the model with outputs [{self.outputs_name}] at {self.tmp_model_path}')
-        self.model = onnxruntime.InferenceSession(self.tmp_model_path, None)
-        self.inputs_name = self.model.get_inputs()[0].name
-        self._device = None
-        self.to_device(self.model)
+        if tmp_model_path and not os.path.exists(tmp_model_path) and self.outputs_name:
+            self._append_outputs(raw_model_path, self.outputs_name, tmp_model_path)
+            self.logger.info(f'save the model with outputs [{self.outputs_name}] at {tmp_model_path}')
+
+        if tmp_model_path and os.path.exists(tmp_model_path):
+            import onnxruntime
+            self.model = onnxruntime.InferenceSession(tmp_model_path, None)
+            self.inputs_name = self.model.get_inputs()[0].name
+            self._device = None
+            self.to_device(self.model)
+        else:
+            raise PretrainedModelFileDoesNotExist(f'model at {tmp_model_path} does not exist')
 
     @staticmethod
     def _append_outputs(input_fn, outputs_name_to_append, output_fn):
@@ -93,7 +99,8 @@ class BaseMindsporeEncoder(MindsporeDevice, BaseEncoder):
                 return YourAwesomeModel()
 
     """
-    def __init__(self, model_path: str, *args, **kwargs):
+
+    def __init__(self, model_path: str = None, *args, **kwargs):
         """
 
         :param model_path: the path of the model's checkpoint.
@@ -107,9 +114,12 @@ class BaseMindsporeEncoder(MindsporeDevice, BaseEncoder):
         """
         from mindspore.train.serialization import load_checkpoint, load_param_into_net
         super().post_init()
-        self.to_device()
-        _param_dict = load_checkpoint(ckpt_file_name=self.model_path)
-        load_param_into_net(self.model, _param_dict)
+        if self.model_path and os.path.exists(self.model_path):
+            self.to_device()
+            _param_dict = load_checkpoint(ckpt_file_name=self.model_path)
+            load_param_into_net(self.model, _param_dict)
+        else:
+            raise PretrainedModelFileDoesNotExist(f'model {self.model_path} does not exist')
 
     @cached_property
     def model(self):
