@@ -1,26 +1,29 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+import argparse
+import base64
 import copy
 import os
 import tempfile
 import threading
 import time
-from collections import OrderedDict, defaultdict, deque
+from collections import OrderedDict, defaultdict
 from contextlib import ExitStack
-from functools import wraps
-from typing import Union, Tuple, List, Set, Dict, Iterator, Callable, Type, TextIO, Any
+from typing import Optional, Union, Tuple, List, Set, Dict, Iterator, Callable, Type, TextIO, Any
+from urllib.request import Request, urlopen
 
 import ruamel.yaml
 from ruamel.yaml import StringIO
 
+from .builder import build_required, _build_flow, _optimize_flow
 from .. import JINA_GLOBAL
-from ..enums import FlowBuildLevel, FlowOptimizeLevel
-from ..excepts import FlowTopologyError, FlowMissingPodError, FlowBuildLevelError
+from ..enums import FlowBuildLevel, PodRoleType, FlowInspectType
+from ..excepts import FlowTopologyError, FlowMissingPodError
 from ..helper import yaml, expand_env_var, get_non_defaults_args, deprecated_alias, complete_path
-from ..logging import get_logger
+from ..logging import JinaLogger
 from ..logging.sse import start_sse_logger
-from ..peapods.pod import SocketType, FlowPod, GatewayFlowPod
+from ..peapods.pod import FlowPod, GatewayFlowPod
 
 if False:
     from ..proto import jina_pb2
@@ -28,123 +31,8 @@ if False:
     import numpy as np
 
 
-def build_required(required_level: 'FlowBuildLevel'):
-    """Annotate a function so that it requires certain build level to run.
-
-    :param required_level: required build level to run this function.
-
-    Example:
-
-    .. highlight:: python
-    .. code-block:: python
-
-        @build_required(FlowBuildLevel.RUNTIME)
-        def foo():
-            print(1)
-
-    """
-
-    def __build_level(func):
-        @wraps(func)
-        def arg_wrapper(self, *args, **kwargs):
-            if hasattr(self, '_build_level'):
-                if self._build_level.value >= required_level.value:
-                    return func(self, *args, **kwargs)
-                else:
-                    raise FlowBuildLevelError(
-                        'build_level check failed for %r, required level: %s, actual level: %s' % (
-                            func, required_level, self._build_level))
-            else:
-                raise AttributeError(f'{self!r} has no attribute "_build_level"')
-
-        return arg_wrapper
-
-    return __build_level
-
-
-def _traverse_graph(op_flow: 'Flow', outgoing_map: Dict[str, List[str]],
-                    func: Callable[['Flow', str, str], None]) -> 'Flow':
-    _outgoing_idx = dict.fromkeys(outgoing_map.keys(), 0)
-    stack = deque()
-    stack.append('gateway')
-    op_flow.logger.debug('Traversing dependency graph:')
-    while stack:
-        start_node_name = stack.pop()
-        end_node_idx = _outgoing_idx[start_node_name]
-        if end_node_idx < len(outgoing_map[start_node_name]):
-            # else, you are back to the gateway
-            end_node_name = outgoing_map[start_node_name][end_node_idx]
-            func(op_flow, start_node_name, end_node_name)
-            stack.append(end_node_name)
-            if end_node_idx + 1 < len(outgoing_map[start_node_name]):
-                stack.append(start_node_name)
-            _outgoing_idx[start_node_name] = end_node_idx + 1
-    return op_flow
-
-
-def _build_flow(op_flow: 'Flow', outgoing_map: Dict[str, List[str]]) -> 'Flow':
-    def _build_two_connections(flow: 'Flow', start_node_name: str, end_node_name: str):
-        # Rule
-        # if a node has multiple income/outgoing peas,
-        # then its socket_in/out must be PULL_BIND or PUB_BIND
-        # otherwise it should be different than its income
-        # i.e. income=BIND => this=CONNECT, income=CONNECT => this = BIND
-        #
-        # when a socket is BIND, then host must NOT be set, aka default host 0.0.0.0
-        # host_in and host_out is only set when corresponding socket is CONNECT
-        start_node = flow._pod_nodes[start_node_name]
-        end_node = flow._pod_nodes[end_node_name]
-        first_socket_type = SocketType.PUSH_CONNECT
-        if len(outgoing_map[start_node_name]) > 1:
-            first_socket_type = SocketType.PUB_BIND
-        elif end_node_name == 'gateway':
-            first_socket_type = SocketType.PUSH_BIND
-        flow.logger.debug(f'Connect {start_node_name} with {end_node_name}')
-        FlowPod.connect(start_node, end_node, first_socket_type=first_socket_type)
-
-    return _traverse_graph(op_flow, outgoing_map, _build_two_connections)
-
-
-def _optimize_flow(op_flow, outgoing_map: Dict[str, List[str]], pod_edges: {str, str}) -> 'Flow':
-    def _optimize_two_connections(flow: 'Flow', start_node_name: str, end_node_name: str):
-        start_node = flow._pod_nodes[start_node_name]
-        end_node = flow._pod_nodes[end_node_name]
-        edges_with_same_start = [ed for ed in pod_edges if ed[0].startswith(start_node_name)]
-        edges_with_same_end = [ed for ed in pod_edges if ed[1].endswith(end_node_name)]
-        if len(edges_with_same_start) > 1 or len(edges_with_same_end) > 1:
-            flow.logger.info(f'Connection between {start_node_name} and {end_node_name} cannot be optimized')
-        else:
-            if start_node_name == 'gateway':
-                if flow.args.optimize_level > FlowOptimizeLevel.IGNORE_GATEWAY and end_node.is_head_router:
-                    flow.logger.info(
-                        f'Node {end_node_name} connects to tail of {start_node_name}')
-                    end_node.connect_to_tail_of(start_node)
-            elif end_node_name == 'gateway':
-                if flow.args.optimize_level > FlowOptimizeLevel.IGNORE_GATEWAY and \
-                        start_node.is_tail_router and start_node.tail_args.num_part <= 1:
-                    # connect gateway directly to peas only if this is unblock router
-                    # as gateway can not block & reduce message
-                    flow.logger.info(
-                        f'Node {start_node_name} connects to head of {end_node_name}')
-                    start_node.connect_to_head_of(end_node)
-            else:
-                if end_node.is_head_router and not start_node.is_tail_router:
-                    flow.logger.info(
-                        f'Node {end_node_name} connects to tail of {start_node_name}')
-                    end_node.connect_to_tail_of(start_node)
-                elif start_node.is_tail_router and start_node.tail_args.num_part <= 1:
-                    flow.logger.info(
-                        f'Node {start_node_name} connects to head of {end_node_name}')
-                    start_node.connect_to_head_of(end_node)
-
-    if op_flow.args.optimize_level > FlowOptimizeLevel.NONE:
-        return _traverse_graph(op_flow, outgoing_map, _optimize_two_connections)
-    else:
-        return op_flow
-
-
 class Flow(ExitStack):
-    def __init__(self, args: 'argparse.Namespace' = None, **kwargs):
+    def __init__(self, args: Optional['argparse.Namespace'] = None, **kwargs):
         """Initialize a flow object
 
         :param kwargs: other keyword arguments that will be shared by all pods in this flow
@@ -162,15 +50,17 @@ class Flow(ExitStack):
         The optimized version, i.e. :code:`Flow(optimize_level=FlowOptimizeLevel.FULL)`
         will generate 4 Peas, but it will force the :class:`GatewayPea` to take BIND role,
         as the head and tail routers are removed.
-        
+
         """
         super().__init__()
-        self.logger = get_logger(self.__class__.__name__)
+        if isinstance(args, argparse.Namespace):
+            self.logger = JinaLogger(self.__class__.__name__, **vars(args))
+        else:
+            self.logger = JinaLogger(self.__class__.__name__)
         self._pod_nodes = OrderedDict()  # type: Dict[str, 'FlowPod']
+        self._inspect_pods = {}  # type: Dict[str, str]
         self._build_level = FlowBuildLevel.EMPTY
-        self._pod_name_counter = 0
         self._last_changed_pod = ['gateway']  #: default first pod is gateway, will add when build()
-
         self._update_args(args, **kwargs)
 
     def _update_args(self, args, **kwargs):
@@ -181,8 +71,6 @@ class Flow(ExitStack):
             _, args, _ = get_parsed_args(kwargs, _flow_parser, 'Flow')
 
         self.args = args
-        if kwargs and self.args.logserver and 'log_sse' not in kwargs:
-            kwargs['log_sse'] = True
         self._common_kwargs = kwargs
         self._kwargs = get_non_defaults_args(args, _flow_parser)  #: for yaml dump
 
@@ -305,7 +193,7 @@ class Flow(ExitStack):
             endpoint = [endpoint]
         elif not endpoint:
             if op_flow._last_changed_pod and connect_to_last_pod:
-                endpoint = [op_flow._last_changed_pod[-1]]
+                endpoint = [op_flow.last_pod]
             else:
                 endpoint = []
 
@@ -315,9 +203,17 @@ class Flow(ExitStack):
                     raise FlowTopologyError('the income/output of a pod can not be itself')
         else:
             raise ValueError(f'endpoint={endpoint} is not parsable')
-        return set(endpoint)
 
-    def set_last_pod(self, name: str, copy_flow: bool = True) -> 'Flow':
+        # if an endpoint is being inspected, then replace it with inspected Pod
+        endpoint = set(op_flow._inspect_pods.get(ep, ep) for ep in endpoint)
+        return endpoint
+
+    @property
+    def last_pod(self):
+        return self._last_changed_pod[-1]
+
+    @last_pod.setter
+    def last_pod(self, name: str):
         """
         Set a pod as the last pod in the flow, useful when modifying the flow.
 
@@ -325,21 +221,17 @@ class Flow(ExitStack):
         :param copy_flow: when set to true, then always copy the current flow and do the modification on top of it then return, otherwise, do in-line modification
         :return: a (new) flow object with modification
         """
-        op_flow = copy.deepcopy(self) if copy_flow else self
-
-        if name not in op_flow._pod_nodes:
+        if name not in self._pod_nodes:
             raise FlowMissingPodError(f'{name} can not be found in this Flow')
 
-        if op_flow._last_changed_pod and name == op_flow._last_changed_pod[-1]:
+        if self._last_changed_pod and name == self.last_pod:
             pass
         else:
-            op_flow._last_changed_pod.append(name)
+            self._last_changed_pod.append(name)
 
         # graph is now changed so we need to
         # reset the build level to the lowest
-        op_flow._build_level = FlowBuildLevel.EMPTY
-
-        return op_flow
+        self._build_level = FlowBuildLevel.EMPTY
 
     def _add_gateway(self, needs, **kwargs):
         pod_name = 'gateway'
@@ -348,8 +240,8 @@ class Flow(ExitStack):
         kwargs['name'] = 'gateway'
         self._pod_nodes[pod_name] = GatewayFlowPod(kwargs, needs)
 
-    def needs(self, needs: Union[Tuple[str], List[str]], uses: str = '_merge', name: str = 'joiner', *args,
-              copy_flow: bool = True, **kwargs) -> 'Flow':
+    def needs(self, needs: Union[Tuple[str], List[str]],
+              uses: str = '_merge', name: str = 'joiner', *args, **kwargs) -> 'Flow':
         """
         Add a blocker to the flow, wait until all peas defined in **needs** completed.
 
@@ -358,15 +250,14 @@ class Flow(ExitStack):
         :param name: the name of this joiner, by default is ``joiner``
         :return: the modified flow
         """
-        op_flow = copy.deepcopy(self) if copy_flow else self
-
         if len(needs) <= 1:
             raise FlowTopologyError('no need to wait for a single service, need len(needs) > 1')
-        return op_flow.add(name=name, uses=uses, needs=needs, *args, **kwargs)
+        return self.add(name=name, uses=uses, needs=needs, pod_role=PodRoleType.JOIN, *args, **kwargs)
 
     def add(self,
             needs: Union[str, Tuple[str], List[str]] = None,
             copy_flow: bool = True,
+            pod_role: 'PodRoleType' = PodRoleType.POD,
             **kwargs) -> 'Flow':
         """
         Add a pod to the current flow object and return the new modified flow object.
@@ -378,6 +269,7 @@ class Flow(ExitStack):
 
         :param needs: the name of the pod(s) that this pod receives data from.
                            One can also use 'pod.Gateway' to indicate the connection with the gateway.
+        :param pod_role: the role of the Pod, used for visualization and route planning
         :param copy_flow: when set to true, then always copy the current flow and do the modification on top of it then return, otherwise, do in-line modification
         :param kwargs: other keyword-value arguments that the pod CLI supports
         :return: a (new) flow object with modification
@@ -388,11 +280,12 @@ class Flow(ExitStack):
         pod_name = kwargs.get('name', None)
 
         if pod_name in op_flow._pod_nodes:
-            raise FlowTopologyError(f'name: {pod_name} is used in this Flow already!')
+            new_name = f'{pod_name}{len(op_flow._pod_nodes)}'
+            self.logger.debug(f'"{pod_name}" is used in this Flow already! renamed it to "{new_name}"')
+            pod_name = new_name
 
         if not pod_name:
-            pod_name = '%s%d' % ('pod', op_flow._pod_name_counter)
-            op_flow._pod_name_counter += 1
+            pod_name = f'pod{len(op_flow._pod_nodes)}'
 
         if not pod_name.isidentifier():
             # hyphen - can not be used in the name
@@ -402,11 +295,84 @@ class Flow(ExitStack):
 
         kwargs.update(op_flow._common_kwargs)
         kwargs['name'] = pod_name
-        op_flow._pod_nodes[pod_name] = FlowPod(kwargs=kwargs, needs=needs)
-        op_flow.set_last_pod(pod_name, False)
+
+        op_flow._pod_nodes[pod_name] = FlowPod(kwargs=kwargs, needs=needs, pod_role=pod_role)
+        op_flow.last_pod = pod_name
 
         return op_flow
 
+    def inspect(self, name: str = 'inspect', *args, **kwargs) -> 'Flow':
+        """Add an inspection on the last changed Pod in the Flow
+
+        Internally, it adds two pods to the flow. But no worry, the overhead is minimized and you
+        can remove them by simply give `Flow(inspect=FlowInspectType.REMOVE)` before using the flow.
+
+        .. highlight:: bash
+        .. code-block:: bash
+
+            Flow -- PUB-SUB -- BasePod(_pass) -- Flow
+                    |
+                    -- PUB-SUB -- InspectPod (Hanging)
+
+        In this way, :class:`InspectPod` looks like a simple ``_pass`` from outside and
+        does not introduce side-effect (e.g. changing the socket type) to the original flow.
+        The original incoming and outgoing socket types are preserved.
+
+        This function is very handy for introducing evaluator into the flow.
+
+        .. seealso::
+
+            :meth:`gather_inspect`
+
+        """
+
+        _last_pod = self.last_pod
+        op_flow = self.add(name=name, needs=_last_pod, pod_role=PodRoleType.INSPECT, *args, **kwargs)
+
+        # now remove uses and add an auxiliary Pod
+        if 'uses' in kwargs:
+            kwargs.pop('uses')
+        op_flow = op_flow.add(name=f'_aux_{name}', uses='_pass', needs=_last_pod,
+                              pod_role=PodRoleType.INSPECT_AUX_PASS, *args, **kwargs)
+
+        # register any future connection to _last_pod by the auxiliary pod
+        op_flow._inspect_pods[_last_pod] = op_flow.last_pod
+
+        return op_flow
+
+    def gather_inspect(self, name: str = 'gather_inspect', uses='_merge_eval', include_last_pod: bool = True, *args,
+                       **kwargs) -> 'Flow':
+        """ Gather all inspect pods output into one pod. When the flow has no inspect pod then the flow itself
+        is returned.
+
+        .. note::
+
+            If ``--no-inspect`` is **not** given, then :meth:`gather_inspect` is auto called before :meth:`build`. So
+            in general you don't need to manually call :meth:`gather_inspect`.
+
+        :param name: the name of the gather pod
+        :param uses: the config of the executor, by default is ``_merge``
+        :param include_last_pod: if to include the last modified pod in the flow
+        :param args:
+        :param kwargs:
+        :return: the modified flow or the copy of it
+
+
+        .. seealso::
+
+            :meth:`inspect`
+
+        """
+
+        needs = [k for k, v in self._pod_nodes.items() if v.role == PodRoleType.INSPECT]
+        if needs:
+            if include_last_pod:
+                needs.append(self.last_pod)
+            return self.add(name=name, uses=uses, needs=needs, pod_role=PodRoleType.JOIN_INSPECT, *args, **kwargs)
+        else:
+            # no inspect node is in the graph, return the current graph
+            return self
+
     def build(self, copy_flow: bool = False) -> 'Flow':
         """
         Build the current flow and make it ready to use
@@ -438,12 +404,28 @@ class Flow(ExitStack):
 
         _pod_edges = set()
 
+        if op_flow.args.inspect == FlowInspectType.COLLECT:
+            op_flow.gather_inspect(copy_flow=False)
+
         if 'gateway' not in op_flow._pod_nodes:
-            op_flow._add_gateway(needs={op_flow._last_changed_pod[-1]})
+            op_flow._add_gateway(needs={op_flow.last_pod})
 
         # construct a map with a key a start node and values an array of its end nodes
         _outgoing_map = defaultdict(list)
+
+        # if set no_inspect then all inspect related nodes are removed
+        if op_flow.args.inspect == FlowInspectType.REMOVE:
+            op_flow._pod_nodes = {k: v for k, v in op_flow._pod_nodes.items() if not v.role.is_inspect}
+            reverse_inspect_map = {v: k for k, v in op_flow._inspect_pods.items()}
+
         for end, pod in op_flow._pod_nodes.items():
+            # if an endpoint is being inspected, then replace it with inspected Pod
+            # but not those inspect related node
+            if op_flow.args.inspect.is_keep:
+                pod.needs = set(ep if pod.role.is_inspect else op_flow._inspect_pods.get(ep, ep) for ep in pod.needs)
+            else:
+                pod.needs = set(reverse_inspect_map.get(ep, ep) for ep in pod.needs)
+
             for start in pod.needs:
                 if start not in op_flow._pod_nodes:
                     raise FlowMissingPodError(f'{start} is not in this flow, misspelled name?')
@@ -468,11 +450,12 @@ class Flow(ExitStack):
         self._build_level = FlowBuildLevel.EMPTY
         self.logger.success(
             f'flow is closed and all resources should be released already, current build level is {self._build_level}')
+        self.logger.close()
 
     def _stop_log_server(self):
         import urllib.request
         try:
-            #it may have been shutdown from the outside
+            # it may have been shutdown from the outside
             urllib.request.urlopen(JINA_GLOBAL.logserver.shutdown, timeout=5)
         except Exception as ex:
             self.logger.info(f'Failed to connect to shutdown log sse server: {repr(ex)}')
@@ -710,7 +693,7 @@ class Flow(ExitStack):
                                           **kwargs)
 
     @deprecated_alias(buffer='input_fn', callback='output_fn')
-    def index(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
+    def index(self, input_fn: Union[Iterator[Union['jina_pb2.Document', bytes]], Callable] = None,
               output_fn: Callable[['jina_pb2.Message'], None] = None,
               **kwargs):
         """Do indexing on the current flow
@@ -750,7 +733,7 @@ class Flow(ExitStack):
         self._get_client(**kwargs).index(input_fn, output_fn, **kwargs)
 
     @deprecated_alias(buffer='input_fn', callback='output_fn')
-    def search(self, input_fn: Union[Iterator['jina_pb2.Document'], Iterator[bytes], Callable] = None,
+    def search(self, input_fn: Union[Iterator[Union['jina_pb2.Document', bytes]], Callable] = None,
                output_fn: Callable[['jina_pb2.Message'], None] = None,
                **kwargs):
         """Do searching on the current flow
@@ -790,6 +773,151 @@ class Flow(ExitStack):
         """
         self._get_client(**kwargs).search(input_fn, output_fn, **kwargs)
 
+    def plot(self, output: str = None,
+             vertical_layout: bool = False,
+             inline_display: bool = True,
+             build: bool = True,
+             copy_flow: bool = True) -> 'Flow':
+        """
+        Visualize the flow up to the current point
+        If a file name is provided it will create a jpg image with that name,
+        otherwise it will display the URL for mermaid.
+        If called within IPython notebook, it will be rendered inline,
+        otherwise an image will be created.
+
+        Example,
+
+        .. highlight:: python
+        .. code-block:: python
+
+            flow = Flow().add(name='pod_a').plot('flow.svg')
+
+        :param output: a filename specifying the name of the image to be created,
+                    the suffix svg/jpg determines the file type of the output image
+        :param vertical_layout: top-down or left-right layout
+        :param inline_display: show image directly inside the Jupyter Notebook
+        :param build: build the flow first before plotting, gateway connection can be better showed
+        :param copy_flow: when set to true, then always copy the current flow and
+                do the modification on top of it then return, otherwise, do in-line modification
+        :return: the flow
+        """
+
+        op_flow = copy.deepcopy(self) if copy_flow else self
+        if build:
+            op_flow.build(False)
+        mermaid_graph = ["%%{init: {'theme': 'base', "
+                         "'themeVariables': { 'primaryColor': '#32C8CD', "
+                         "'edgeLabelBackground':'#fff', 'clusterBkg': '#FFCC66'}}}%%"]
+        mermaid_graph.append('graph TD' if vertical_layout else 'graph LR')
+
+        start_repl = {}
+        end_repl = {}
+        for node, v in op_flow._pod_nodes.items():
+            if not v.is_singleton and v.role != PodRoleType.GATEWAY:
+                mermaid_graph.append(f'subgraph sub_{node} ["{node} ({v._args.parallel})"]')
+                if v.is_head_router:
+                    head_router = node + '_HEAD'
+                    end_repl[node] = (head_router, '((fa:fa-random))')
+                if v.is_tail_router:
+                    tail_router = node + '_TAIL'
+                    start_repl[node] = (tail_router, '((fa:fa-random))')
+
+                p_r = '((%s))'
+                p_e = '[[%s]]'
+                for j in range(v._args.parallel):
+                    r = node + (f'_{j}' if v._args.parallel > 1 else '')
+                    if v.is_head_router:
+                        mermaid_graph.append('\t%s%s:::pea-->%s%s:::pea' % (head_router, p_r % 'head', r, p_e % r))
+                    if v.is_tail_router:
+                        mermaid_graph.append('\t%s%s:::pea-->%s%s:::pea' % (r, p_e % r, tail_router, p_r % 'tail'))
+                mermaid_graph.append('end')
+
+        for node, v in op_flow._pod_nodes.items():
+            ed_str = str(v.head_args.socket_in).split('_')[0]
+            for need in sorted(v.needs):
+                edge_str = ''
+                if need in op_flow._pod_nodes:
+                    st_str = str(op_flow._pod_nodes[need].tail_args.socket_out).split('_')[0]
+                    edge_str = f'|{st_str}-{ed_str}|'
+
+                _s = start_repl.get(need, (need, f'({need})'))
+                _e = end_repl.get(node, (node, f'({node})'))
+                _s_role = op_flow._pod_nodes[need].role
+                _e_role = op_flow._pod_nodes[node].role
+                line_st = '-->'
+
+                if _s_role in {PodRoleType.INSPECT, PodRoleType.JOIN_INSPECT}:
+                    _s = start_repl.get(need, (need, f'{{{{{need}}}}}'))
+
+                if _e_role == PodRoleType.GATEWAY:
+                    _e = ('gateway_END', f'({node})')
+                elif _e_role in {PodRoleType.INSPECT, PodRoleType.JOIN_INSPECT}:
+                    _e = end_repl.get(node, (node, f'{{{{{node}}}}}'))
+
+                if _s_role == PodRoleType.INSPECT or _e_role == PodRoleType.INSPECT:
+                    line_st = '-.->'
+
+                mermaid_graph.append(
+                    f'{_s[0]}{_s[1]}:::{str(_s_role)} {line_st} {edge_str}{_e[0]}{_e[1]}:::{str(_e_role)}')
+        mermaid_graph.append(f'classDef {str(PodRoleType.POD)} fill:#32C8CD,stroke:#009999')
+        mermaid_graph.append(f'classDef {str(PodRoleType.INSPECT)} fill:#ff6666,color:#fff')
+        mermaid_graph.append(f'classDef {str(PodRoleType.JOIN_INSPECT)} fill:#ff6666,color:#fff')
+        mermaid_graph.append(f'classDef {str(PodRoleType.GATEWAY)} fill:#6E7278,color:#fff')
+        mermaid_graph.append(f'classDef {str(PodRoleType.INSPECT_AUX_PASS)} fill:#fff,color:#000,stroke-dasharray: 5 5')
+        mermaid_graph.append('classDef pea fill:#009999,stroke:#1E6E73')
+        mermaid_str = '\n'.join(mermaid_graph)
+
+        image_type = 'svg'
+        if output and output.endswith('jpg'):
+            image_type = 'jpg'
+
+        url = op_flow._mermaid_to_url(mermaid_str, image_type)
+        showed = False
+        if inline_display:
+            try:
+                from IPython.display import display, Image
+                from IPython.utils import io
+
+                display(Image(url=url))
+                showed = True
+            except:
+                # no need to panic users
+                pass
+
+        if output:
+            op_flow._download_mermaid_url(url, output)
+        elif not showed:
+            op_flow.logger.info(f'flow visualization: {url}')
+
+        return self
+
+    def _mermaid_to_url(self, mermaid_str, img_type) -> str:
+        """
+        Rendering the current flow as a url points to a SVG, it needs internet connection
+        :param kwargs: keyword arguments of :py:meth:`to_mermaid`
+        :return: the url points to a SVG
+        """
+        if img_type == 'jpg':
+            img_type = 'img'
+
+        encoded_str = base64.b64encode(bytes(mermaid_str, 'utf-8')).decode('utf-8')
+
+        return f'https://mermaid.ink/{img_type}/{encoded_str}'
+
+    def _download_mermaid_url(self, mermaid_url, output) -> None:
+        """
+        Rendering the current flow as a jpg image, this will call :py:meth:`to_mermaid` and it needs internet connection
+        :param path: the file path of the image
+        :param kwargs: keyword arguments of :py:meth:`to_mermaid`
+        :return:
+        """
+        try:
+            req = Request(mermaid_url, headers={'User-Agent': 'Mozilla/5.0'})
+            with open(output, 'wb') as fp:
+                fp.write(urlopen(req).read())
+        except:
+            self.logger.error('can not download image, please check your graph and the network connections')
+
     def dry_run(self, **kwargs):
         """Send a DRYRUN request to this flow, passing through all pods in this flow,
         useful for testing connectivity and debugging"""
