@@ -2,6 +2,9 @@ __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
 import argparse
+import os
+
+_SHOW_ALL_ARGS = 'JINA_FULL_CLI' in os.environ
 
 
 def add_arg_group(parser, title):
@@ -53,7 +56,6 @@ def set_logger_parser(parser=None):
 
 
 def set_hub_base_parser(parser=None):
-    import os
     if not parser:
         parser = set_base_parser()
     parser.add_argument('--username', type=str, help='the registry username',
@@ -125,6 +127,23 @@ def set_hub_pushpull_parser(parser=None):
     return parser
 
 
+def set_hub_list_parser(parser=None):
+    if not parser:
+        parser = set_base_parser()
+
+    parser.add_argument('--name', type=str,
+                        help='name of hub image')
+    parser.add_argument('--kind', type=str,
+                        help='kind of hub image')
+    parser.add_argument('--keywords', type=str, nargs='+',
+                        help='keywords for searching')
+    parser.add_argument('--type', type=str, default='pod', choices=['pod', 'app'],
+                        help='type of the hub image')
+    parser.add_argument('--local-only', action='store_true', default=False,
+                        help='list all local hub images on the current machine')
+    return parser
+
+
 def set_hw_parser(parser=None):
     if not parser:
         parser = set_base_parser()
@@ -151,21 +170,21 @@ def set_hw_parser(parser=None):
                     default=2,
                     help='number of parallel when index and query')
     gp = add_arg_group(parser, 'index arguments')
-    gp.add_argument('--index-uses', type=str,
+    gp.add_argument('--uses-index', type=str,
                     default=resource_filename('jina', '/'.join(('resources', 'helloworld.flow.index.yml'))),
                     help='the yaml path of the index flow')
     gp.add_argument('--index-data-url', type=str,
-                    default='http://fashion-mnist.s3-website.eu-central-1.amazonaws.com/train-images-idx3-ubyte.gz',
+                    default='https://github.com/zalandoresearch/fashion-mnist/raw/master/data/fashion/train-images-idx3-ubyte.gz',
                     help='the url of index data (should be in idx3-ubyte.gz format)')
     gp.add_argument('--index-batch-size', type=int,
                     default=1024,
                     help='the batch size in indexing')
     gp = add_arg_group(parser, 'query arguments')
-    gp.add_argument('--query-uses', type=str,
+    gp.add_argument('--uses-query', type=str,
                     default=resource_filename('jina', '/'.join(('resources', 'helloworld.flow.query.yml'))),
                     help='the yaml path of the query flow')
     gp.add_argument('--query-data-url', type=str,
-                    default='http://fashion-mnist.s3-website.eu-central-1.amazonaws.com/t10k-images-idx3-ubyte.gz',
+                    default='https://github.com/zalandoresearch/fashion-mnist/raw/master/data/fashion/t10k-images-idx3-ubyte.gz',
                     help='the url of query data (should be in idx3-ubyte.gz format)')
     gp.add_argument('--query-batch-size', type=int,
                     default=32,
@@ -181,7 +200,7 @@ def set_hw_parser(parser=None):
 def set_flow_parser(parser=None):
     if not parser:
         parser = set_base_parser()
-    from .enums import FlowOutputType, FlowOptimizeLevel
+    from .enums import FlowOutputType, FlowOptimizeLevel, FlowInspectType
 
     gp = add_arg_group(parser, 'flow arguments')
     gp.add_argument('--uses', type=str, help='a yaml file represents a flow')
@@ -200,6 +219,10 @@ def set_flow_parser(parser=None):
                     help='type of the output')
     gp.add_argument('--output-path', type=argparse.FileType('w', encoding='utf8'),
                     help='output path of the flow')
+    gp.add_argument('--inspect', type=FlowInspectType.from_string,
+                    choices=list(FlowInspectType), default=FlowInspectType.COLLECT,
+                    help='strategy on those inspect pods in the flow. '
+                         'if REMOVE is given then all inspect pods are removed when building the flow')
     return parser
 
 
@@ -207,8 +230,7 @@ def set_pea_parser(parser=None):
     from .enums import SocketType, PeaRoleType, OnErrorSkip
     from .helper import random_port, get_random_identity
     from . import __default_host__
-    import os
-    show_all = 'JINA_FULL_CLI' in os.environ
+
     if not parser:
         parser = set_base_parser()
     gp0 = add_arg_group(parser, 'pea basic arguments')
@@ -279,8 +301,9 @@ def set_pea_parser(parser=None):
     gp3.add_argument('--separated-workspace', action='store_true', default=False,
                      help='the data and config files are separated for each pea in this pod, '
                           'only effective when BasePod\'s `parallel` > 1')
-    gp3.add_argument('--replica-id', type=int, default=-1,
-                     help='the id of the storage of this replica, only effective when `separated_workspace=True`')
+    gp3.add_argument('--pea-id', type=int, default=-1,
+                     help='the id of the storage of this pea, only effective when `separated_workspace=True`'
+                     if _SHOW_ALL_ARGS else argparse.SUPPRESS)
 
     gp5 = add_arg_group(parser, 'pea messaging arguments')
     gp5.add_argument('--check-version', action='store_true', default=False,
@@ -295,13 +318,13 @@ def set_pea_parser(parser=None):
                           'compression rate (after_size/before_size) lower than this LWM will be considered as successeful '
                           'compression, and will be sent. Otherwise, it will send the original message without compression')
     gp5.add_argument('--num-part', type=int, default=0,
-                     **(dict(
-                         help='the number of replicated message sent to the next Pod, 0 and 1 means single part' if show_all else argparse.SUPPRESS)))
+                     help='the number of replicated message sent to the next Pod, 0 and 1 means single part'
+                     if _SHOW_ALL_ARGS else argparse.SUPPRESS)
     gp5.add_argument('--role', type=PeaRoleType.from_string, choices=list(PeaRoleType),
-                     help='the role of this pea in a pod')
+                     help='the role of this pea in a pod' if _SHOW_ALL_ARGS else argparse.SUPPRESS)
     gp5.add_argument('--skip-on-error', type=OnErrorSkip.from_string, choices=list(OnErrorSkip),
                      default=OnErrorSkip.NONE,
-                     help='skip strategy on error message. ')
+                     help='skip strategy on error message.')
 
     gp6 = add_arg_group(parser, 'pea EXPERIMENTAL arguments')
     gp6.add_argument('--memory-hwm', type=int, default=-1,
@@ -316,17 +339,25 @@ def set_pea_parser(parser=None):
                      help='when a process exits, it attempts to terminate all of its daemonic child processes. '
                           'setting it to true basically tell the context manager do not wait on this Pea')
 
+    from pkg_resources import resource_filename
     gp7 = add_arg_group(parser, 'logging arguments')
-    gp7.add_argument('--log-sse', action='store_true', default=False,
-                     help='turn on server-side event logging')
+    gp7.add_argument('--log-config', type=str,
+                     default=resource_filename('jina',
+                                               '/'.join(('resources', 'logging.default.yml'))),
+                     help='the yaml config of the logger. note the executor inside will inherit this log config')
     gp7.add_argument('--log-remote', action='store_true', default=False,
-                     help='turn on remote logging')
-    gp7.add_argument('--log-profile', action='store_true', default=False,
-                     help='turn on the profiling logger')
-    gp7.add_argument('--log-with-own-name', action='store_true', default=False,
-                     help='turn on to let each logger outputs in its own name (i.e. parent class name as the context), '
-                          'by default it is off so all logs from the same pod will have the same prefix. '
-                          'turn on to help debugging, turn off to have more clear logs and better grouping in dashboard')
+                     help='turn on remote logging, this should not be set manually'
+                     if _SHOW_ALL_ARGS else argparse.SUPPRESS)
+
+    gp8 = add_arg_group(parser, 'ssh tunneling arguments')
+    gp8.add_argument('--ssh-server', type=str, default=None,
+                     help='the SSH server through which the tunnel will be created, '
+                          'can actually be a fully specified "user@server:port" ssh url.')
+    gp8.add_argument('--ssh-keyfile', type=str, default=None,
+                     help='this specifies a key to be used in ssh login, default None. '
+                          'regular default ssh keys will be used without specifying this argument.')
+    gp8.add_argument('--ssh-password', type=str, default=None,
+                     help='ssh password to the ssh server.')
     _set_grpc_parser(parser)
     return parser
 
@@ -357,17 +388,6 @@ def set_pod_parser(parser=None):
                           'accepted type follows "--uses"')
     gp4.add_argument('--shutdown-idle', action='store_true', default=False,
                      help='shutdown this pod when all peas are idle')
-
-    # disable the pod level logserver for now
-    # gp5 = add_arg_group(parser, 'pod log-server arguments')
-    #
-    # from pkg_resources import resource_filename
-    # gp5.add_argument('--logserver', action='store_true', default=False,
-    #                  help='start a log server for the dashboard')
-    # gp5.add_argument('--logserver-config', type=str,
-    #                  default=resource_filename('jina',
-    #                                            '/'.join(('resources', 'logserver.default.yml'))),
-    #                  help='the yaml config of the log server')
     return parser
 
 
@@ -427,7 +447,7 @@ def _set_grpc_parser(parser=None):
     gp1.add_argument('--proxy', action='store_true', default=False,
                      help='respect the http_proxy and https_proxy environment variables. '
                           'otherwise, it will unset these proxy variables before start. '
-                          'gRPC seems to prefer --no-proxy')
+                          'gRPC seems to prefer no proxy')
     return parser
 
 
@@ -491,7 +511,6 @@ def set_client_cli_parser(parser=None):
     _set_grpc_parser(parser)
 
     gp1 = add_arg_group(parser, 'client-specific arguments')
-    _gp = gp1.add_mutually_exclusive_group()
 
     gp1.add_argument('--batch-size', type=int, default=100,
                      help='the number of documents in each request')
@@ -499,24 +518,13 @@ def set_client_cli_parser(parser=None):
                      # required=True,
                      help='the mode of the client and the server')
     gp1.add_argument('--top-k', type=int,
-                     default=10,
                      help='top_k results returned in the search mode')
     gp1.add_argument('--mime-type', type=str,
                      help='MIME type of the input, useful when input-type is set to BUFFER')
     gp1.add_argument('--callback-on-body', action='store_true', default=False,
                      help='callback function works directly on the request body')
-    gp1.add_argument('--first-request-id', type=int,
-                     default=0,
-                     help='the starting number of request id, the consequent request_id will increment by one')
-    _gp.add_argument('--first-doc-id', type=int,
-                     default=0,
-                     help='the starting number of the first doc, the consequent id will increment by one')
-    _gp.add_argument('--random-doc-id', action='store_true', default=False,
-                     help='randomize document id, if this is set then `first_request_id` is ignored')
     gp1.add_argument('--timeout-ready', type=int, default=10000,
                      help='timeout (ms) of a pea is ready for request, -1 for waiting forever')
-    gp1.add_argument('--filter-by', type=str, nargs='*',
-                     help='field names to search on')
     gp1.add_argument('--skip-dry-run', action='store_true', default=False,
                      help='skip dry run (connectivity test) before sending every request')
     return parser
@@ -525,8 +533,6 @@ def set_client_cli_parser(parser=None):
 def get_main_parser():
     # create the top-level parser
     parser = set_base_parser()
-    import os
-    show_all = 'JINA_FULL_CLI' in os.environ
 
     sp = parser.add_subparsers(dest='cli',
                                description='use "%(prog)-8s [sub-command] --help" '
@@ -566,6 +572,10 @@ def get_main_parser():
                             description='use "%(prog)-8s [sub-command] --help" '
                                         'to get detailed information about each sub-command', required=True)
 
+    spp.add_parser('login', help='login via Github to push images to Jina hub registry',
+                   description='Login via Github to push images to Jina hub registry',
+                   formatter_class=_chf)
+
     set_hub_new_parser(
         spp.add_parser('new', aliases=['init', 'create'], help='create a new Hub executor or app using cookiecutter',
                        description='Create a new Hub executor or app using cookiecutter',
@@ -586,26 +596,31 @@ def get_main_parser():
                        description='Pull an image to the Jina hub registry to local',
                        formatter_class=_chf))
 
+    set_hub_list_parser(
+        spp.add_parser('list', help='list hub executors from jina hub registry',
+                       description='List hub executors from jina hub registry',
+                       formatter_class=_chf))
+
     set_pea_parser(sp.add_parser('pea',
                                  description='Start a Jina pea. '
                                              'You should rarely use this directly unless you '
                                              'are doing low-level orchestration',
-                                 formatter_class=_chf, **(dict(help='start a pea')) if show_all else {}))
+                                 formatter_class=_chf, **(dict(help='start a pea')) if _SHOW_ALL_ARGS else {}))
 
     set_logger_parser(sp.add_parser('log',
                                     description='Receive piped log output and beautify the log. '
                                                 'Depreciated, use Jina Dashboard instead',
                                     formatter_class=_chf,
-                                    **(dict(help='beautify the log')) if show_all else {}))
+                                    **(dict(help='beautify the log')) if _SHOW_ALL_ARGS else {}))
     set_client_cli_parser(
         sp.add_parser('client',
                       description='Start a Python client that connects to a remote Jina gateway',
-                      formatter_class=_chf, **(dict(help='start a client')) if show_all else {}))
+                      formatter_class=_chf, **(dict(help='start a client')) if _SHOW_ALL_ARGS else {}))
 
     set_export_api_parser(sp.add_parser('export-api',
                                         description='Export Jina API to JSON/YAML file for 3rd party applications',
                                         formatter_class=_chf,
-                                        **(dict(help='export Jina API to file')) if show_all else {}))
+                                        **(dict(help='export Jina API to file')) if _SHOW_ALL_ARGS else {}))
     return parser
 
 
