@@ -13,17 +13,19 @@ from typing import Dict, Optional, Union
 
 import zmq
 
+from jina import __unable_to_load_pretrained_model_msg__
 from .zmq import send_ctrl_message, Zmqlet, ZmqStreamlet
 from .. import __ready_msg__, __stop_msg__
 from ..drivers.helper import routes2str, add_route
 from ..enums import PeaRoleType, OnErrorSkip
-from ..excepts import NoExplicitMessage, ExecutorFailToLoad, MemoryOverHighWatermark, DriverError, PeaFailToStart
+from ..excepts import NoExplicitMessage, ExecutorFailToLoad, MemoryOverHighWatermark, DriverError, PeaFailToStart, \
+    PretrainedModelFileDoesNotExist
 from ..executors import BaseExecutor
 from ..helper import is_valid_local_config_source
-from ..logging import get_logger
+from ..logging import JinaLogger
 from ..logging.profile import used_memory, TimeDict
-from ..proto import jina_pb2
 from ..logging.queue import clear_queues
+from ..proto import jina_pb2
 
 __all__ = ['PeaMeta', 'BasePea']
 
@@ -106,17 +108,18 @@ class BasePea(metaclass=PeaMeta):
         """ Create a new :class:`BasePea` object
 
         :param args: the arguments received from the CLI
-        :param replica_id: the id used to separate the storage of each pea, only used when ``args.separate_storage=True``
+        :param pea_id: the id used to separate the storage of each pea, only used when ``args.separate_storage=True``
         """
         super().__init__()
         self.args = args
         self.name = self.__class__.__name__  #: this is the process name
-        self.daemon = args.daemon
 
         self.is_ready = _get_event(self)
         self.is_shutdown = _get_event(self)
+        self.is_pretrained_model_exception = _get_event(self)
         self.ready_or_shutdown = _make_or_event(self, self.is_ready, self.is_shutdown)
         self.is_shutdown.clear()
+        self.is_pretrained_model_exception.clear()
 
         self.last_active_time = time.perf_counter()
         self.last_dump_time = time.perf_counter()
@@ -127,17 +130,18 @@ class BasePea(metaclass=PeaMeta):
         self._message = None
 
         if isinstance(self.args, argparse.Namespace):
+            self.daemon = args.daemon
             if self.args.name:
                 self.name = self.args.name
-            if self.args.role == PeaRoleType.REPLICA:
-                self.name = '%s-%d' % (self.name, self.args.replica_id)
+            if self.args.role == PeaRoleType.PARALLEL:
+                self.name = '%s-%d' % (self.name, self.args.pea_id)
             self.ctrl_addr, self.ctrl_with_ipc = Zmqlet.get_ctrl_address(self.args)
-            if not self.args.log_with_own_name and self.args.name:
+            if self.args.name:
                 # everything in this Pea (process) will use the same name for display the log
                 os.environ['JINA_POD_NAME'] = self.args.name
-            self.logger = get_logger(self.name, **vars(self.args))
+            self.logger = JinaLogger(self.name, **vars(self.args))
         else:
-            self.logger = get_logger(self.name)
+            self.logger = JinaLogger(self.name)
 
     def __str__(self):
         r = self.name
@@ -193,10 +197,13 @@ class BasePea(metaclass=PeaMeta):
             try:
                 self.executor = BaseExecutor.load_config(
                     self.args.uses if is_valid_local_config_source(self.args.uses) else self.args.uses_internal,
-                    self.args.separated_workspace, self.args.replica_id)
+                    self.args.separated_workspace, self.args.pea_id)
                 self.executor.attach(pea=self)
             except FileNotFoundError:
                 raise ExecutorFailToLoad
+            except PretrainedModelFileDoesNotExist as exception:
+                self.is_pretrained_model_exception.set()
+                raise exception
         else:
             self.logger.warning('this BasePea has no executor attached, you may want to double-check '
                                 'if it is a mistake or on purpose (using this BasePea as router/map-reduce)')
@@ -349,6 +356,9 @@ class BasePea(metaclass=PeaMeta):
             self.logger.critical(f'driver error: {repr(ex)}', exc_info=True)
         except zmq.error.ZMQError:
             self.logger.critical('zmqlet can not be initiated')
+        except PretrainedModelFileDoesNotExist:
+            self.logger.critical(__unable_to_load_pretrained_model_msg__)
+            self.is_pretrained_model_exception.set()
         except Exception as ex:
             # this captures the general exception from the following places:
             # - self.zmqlet.recv_message
@@ -402,7 +412,10 @@ class BasePea(metaclass=PeaMeta):
                 # return too early and the shutdown is set, means something fails!!
                 self.logger.critical(f'fail to start {self.__class__} with name {self.name}, '
                                      f'this often means the executor used in the pod is not valid')
-                raise PeaFailToStart
+                if self.is_pretrained_model_exception.is_set():
+                    raise PretrainedModelFileDoesNotExist
+                else:
+                    raise PeaFailToStart
             return self
         else:
             raise TimeoutError(
@@ -416,6 +429,7 @@ class BasePea(metaclass=PeaMeta):
         self.is_shutdown.wait()
         if not self.daemon:
             clear_queues()
+            self.logger.close()
             self.join()
 
     def __exit__(self, exc_type, exc_val, exc_tb) -> None:
