@@ -18,7 +18,7 @@ from .. import __default_host__
 from ..enums import *
 from ..helper import random_port, get_random_identity, get_parsed_args, get_non_defaults_args, \
     is_valid_local_config_source
-from ..main.parser import set_pod_parser, set_gateway_parser
+from ..parser import set_pod_parser, set_gateway_parser
 
 
 class BasePod(ExitStack):
@@ -37,17 +37,14 @@ class BasePod(ExitStack):
         self.is_tail_router = False
         self.deducted_head = None
         self.deducted_tail = None
-        if hasattr(args, 'polling') and args.polling.is_push:
-            # ONLY reset when it is push
-            args.uses_after = '_pass'
-
-        if getattr(args, 'parallel', 1) > 1:
-            self.is_head_router = True
-            self.is_tail_router = True
-
         self._args = args
         self.peas_args = self._parse_args(args)
 
+    @property
+    def is_singleton(self) -> bool:
+        """Return if the Pod contains only a single Pea """
+        return not (self.is_head_router or self.is_tail_router)
+
     @property
     def is_idle(self) -> bool:
         """A Pod is idle when all its peas are idle, see also :attr:`jina.peapods.pea.Pea.is_idle`.
@@ -82,16 +79,31 @@ class BasePod(ExitStack):
             'tail': None,
             'peas': []
         }
-
         if getattr(args, 'parallel', 1) > 1:
             # reasons to separate head and tail from peas is that they
             # can be deducted based on the previous and next pods
+            _set_after_to_pass(args)
+            self.is_head_router = True
+            self.is_tail_router = True
             peas_args['head'] = _copy_to_head_args(args, args.polling.is_push)
             peas_args['tail'] = _copy_to_tail_args(args)
             peas_args['peas'] = _set_peas_args(args, peas_args['head'], peas_args['tail'])
+        elif getattr(args, 'uses_before', None) or getattr(args, 'uses_after', None):
+            args.scheduling = SchedulerType.ROUND_ROBIN
+            if getattr(args, 'uses_before', None):
+                self.is_head_router = True
+                peas_args['head'] = _copy_to_head_args(args, args.polling.is_push)
+            if getattr(args, 'uses_after', None):
+                self.is_tail_router = True
+                peas_args['tail'] = _copy_to_tail_args(args)
+            peas_args['peas'] = _set_peas_args(args, peas_args.get('head', None), peas_args.get('tail', None))
         else:
+            _set_after_to_pass(args)
+            self.is_head_router = False
+            self.is_tail_router = False
             peas_args['peas'] = [args]
 
+
         # note that peas_args['peas'][0] exist either way and carries the original property
         return peas_args
 
@@ -200,12 +212,12 @@ class BasePod(ExitStack):
         # start real peas and accumulate the storage id
         if len(self.peas_args['peas']) > 1:
             start_rep_id = 1
-            role = PeaRoleType.REPLICA
+            role = PeaRoleType.PARALLEL
         else:
             start_rep_id = 0
             role = PeaRoleType.SINGLETON
         for idx, _args in enumerate(self.peas_args['peas'], start=start_rep_id):
-            _args.replica_id = idx
+            _args.pea_id = idx
             _args.role = role
             p = Pea(_args, allow_remote=False)
             self.peas.append(p)
@@ -278,11 +290,16 @@ class MutablePod(BasePod):
 
 class FlowPod(BasePod):
     """A :class:`FlowPod` is like a :class:`BasePod`, but it exposes more interfaces for tweaking its connections with
-    other Pods, which comes in handy when used in the Flow API
+    other Pods, which comes in handy when used in the Flow API.
+
+    .. note::
+
+        Unlike :class:`BasePod`, this class takes a :class:`dict` as the first argument.
+
     """
 
     def __init__(self, kwargs: Dict,
-                 needs: Set[str] = None, parser: Callable = set_pod_parser):
+                 needs: Set[str] = None, parser: Callable = set_pod_parser, pod_role: 'PodRoleType' = PodRoleType.POD):
         """
 
         :param kwargs: unparsed argument in dict, if given the
@@ -293,6 +310,7 @@ class FlowPod(BasePod):
         super().__init__(self._args)
         self.needs = needs if needs else set()  #: used in the :class:`jina.flow.Flow` to build the graph
         self._kwargs = get_non_defaults_args(self._args, _parser)
+        self.role = pod_role
 
     def to_cli_command(self):
         if isinstance(self, GatewayPod):
@@ -376,12 +394,14 @@ class FlowPod(BasePod):
             return self
 
 
-def _set_peas_args(args: Namespace, head_args: Namespace, tail_args: Namespace) -> List[Namespace]:
+def _set_peas_args(args: Namespace, head_args: Namespace = None, tail_args: Namespace = None) -> List[Namespace]:
     result = []
     for _ in range(args.parallel):
         _args = copy.deepcopy(args)
-        _args.port_in = head_args.port_out
-        _args.port_out = tail_args.port_in
+        if head_args:
+            _args.port_in = head_args.port_out
+        if tail_args:
+            _args.port_out = tail_args.port_in
         _args.port_ctrl = random_port()
         _args.identity = get_random_identity()
         _args.socket_out = SocketType.PUSH_CONNECT
@@ -394,12 +414,22 @@ def _set_peas_args(args: Namespace, head_args: Namespace, tail_args: Namespace)
                 raise NotImplementedError
         else:
             _args.socket_in = SocketType.SUB_CONNECT
-        _args.host_in = _fill_in_host(bind_args=head_args, connect_args=_args)
-        _args.host_out = _fill_in_host(bind_args=tail_args, connect_args=_args)
+        if head_args:
+            _args.host_in = _fill_in_host(bind_args=head_args, connect_args=_args)
+        if tail_args:
+            _args.host_out = _fill_in_host(bind_args=tail_args, connect_args=_args)
         result.append(_args)
     return result
 
 
+def _set_after_to_pass(args):
+    # TODO: I don't remember what is this for? once figure out, this function should be removed
+    # remark 1: i think it's related to route driver.
+    if hasattr(args, 'polling') and args.polling.is_push:
+        # ONLY reset when it is push
+        args.uses_after = '_pass'
+
+
 def _copy_to_head_args(args: Namespace, is_push: bool, as_router: bool = True) -> Namespace:
     """Set the outgoing args of the head router
     """
@@ -438,6 +468,7 @@ def _copy_to_tail_args(args: Namespace, as_router: bool = True) -> Namespace:
     _tail_args.port_ctrl = random_port()
     _tail_args.socket_in = SocketType.PULL_BIND
     _tail_args.uses = None
+
     if as_router:
         _tail_args.uses = args.uses_after or '_merge'
         _tail_args.name = args.name or ''
@@ -452,7 +483,7 @@ def _fill_in_host(bind_args: Namespace, connect_args: Namespace) -> str:
     bind_local = (bind_args.host == '0.0.0.0')
     conn_local = (connect_args.host == '0.0.0.0')
     conn_docker = (
-                getattr(connect_args, 'uses', None) is not None and not is_valid_local_config_source(connect_args.uses))
+            getattr(connect_args, 'uses', None) is not None and not is_valid_local_config_source(connect_args.uses))
     bind_conn_same_remote = not bind_local and not conn_local and (bind_args.host == connect_args.host)
     if platform == "linux" or platform == "linux2":
         local_host = '0.0.0.0'
@@ -490,3 +521,4 @@ class GatewayFlowPod(GatewayPod, FlowPod):
 
     def __init__(self, kwargs: Dict = None, needs: Set[str] = None):
         FlowPod.__init__(self, kwargs, needs, parser=set_gateway_parser)
+        self.role = PodRoleType.GATEWAY
